<?php

        // autoload dependencies
        require __DIR__ . '/vendor/autoload.php';


	/**
	 *
	 */
	function plugin_CereusTransporter_install()
	{
		api_plugin_register_hook( 'CereusTransporter', 'poller_top', 'CereusTransporter_poller_top', 'setup.php' );
		api_plugin_register_hook( 'CereusTransporter', 'poller_bottom', 'CereusTransporter_poller_bottom', 'setup.php' );
		api_plugin_register_hook( 'CereusTransporter', 'poller_output', 'CereusTransporter_poller_output', 'setup.php' );
		api_plugin_register_hook( 'CereusTransporter', 'console_after', 'CereusTransporter_console_after', 'setup.php' );
		api_plugin_register_hook( 'CereusTransporter', 'config_settings', 'CereusTransporter_config_settings', 'setup.php' );
		CereusTransporter_setup_table_new();
	}

	function CereusTransporter_poller_top() {
		if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
			cacti_log( "DEBUG: Preparing for the next pass, deleting all data from plugin_CereusTransporter_data (if there was any)", TRUE, "CereusTransporter" );
		}
		db_execute( "DELETE FROM `plugin_CereusTransporter_data`" );
	}

	/**
	 * @param $rrd_update_array
	 *
	 * @return mixed
	 */
	function CereusTransporter_poller_output( $rrd_update_array )
	{
		foreach($rrd_update_array as $item) {
			if ( is_array($item) ) {
				if ( array_key_exists('times',$item) ) {
					$array = $item['times'][key($item['times'])];
					while( list ( $key, $val ) = each ( $array ) ) {
						//cacti_log("Poller Item : ".$item['local_data_id']."[ $key -> $val ]");
						if ( strlen($key) > 0 ) {
							db_execute("INSERT INTO `plugin_CereusTransporter_data` (`timestamp`, `local_data_id`, `key`, `value`) VALUES ('".key($item['times'])."',".$item['local_data_id'].",'".$key."','".$val."') ");
						}
					}
				}
			}
		}
		return $rrd_update_array;
	}
	
	function CereusTransporter_send_data($data_array) {
		$db_url = read_config_option( 'cereus_transporter_db_fullurl' );
		$db_type = read_config_option( 'cereus_transporter_dbtype' );

		// curl for bosun and opentsdb
		$curl = null;

		// init request
		switch ($db_type) {
			case 'influxdb':
				// autoload dependencies ( namely InfluxDB client )
				require_once __DIR__ . '/vendor/autoload.php';
				// Connect to the remote InfluxDB host and fetch the database
				try {
					$database = InfluxDB\Client::fromDSN($db_url);
				} catch  (Exception $e) {
					cacti_log( "ERROR: ".$e->getMessage(), TRUE, "CereusTransporter" );
				}
				break;
			case 'opentsdb':
			case 'bosun':
				$curl = curl_init();
				$db_url = rtrim($db_url, '/');
				$page = '/api/put?details';
				if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
					$page .= '?details';
				}
				curl_setopt( $curl, CURLOPT_URL, $db_url.$page );
				curl_setopt( $curl, CURLOPT_RETURNTRANSFER, TRUE );
				curl_setopt( $curl, CURLOPT_BINARYTRANSFER, TRUE );
				curl_setopt( $curl, CURLOPT_POST, TRUE );
			break;
		}

		$metrics = array();

		// preparing points
		$points = array();
		foreach ( $data_array as $point ) {
			if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
				cacti_log( "DEBUG: Appending the following data to request: [".$db_type."] [".$point['timestamp']."] [".$point['tags']['hostname']."] [".json_encode($point['tags'])."] [".$point['tags']['type']."] [".$point['value']."]", TRUE, "CereusTransporter" );
			}
			if ( strlen( $point['metric'] ) > 0 ) {
				switch ( $db_type ) {
					case 'influxdb':
						try {
							$points[] = new InfluxDB\Point(
								$point['metric'], 
								$point['value'], 
								$point['tags'], 
								array('value' => $point['value']), 
								$point['timestamp']);
						} catch  (Exception $e) {
							cacti_log( "ERROR: ".$e->getMessage(), TRUE, "CereusTransporter" );
						}
					break;
					case 'opentsdb':
					case 'bosun':
						// prepare metrics info for bosun
						if ( $db_type == 'bosun' && is_null( $metrics[$point['metric']] ) ) {
							$metrics_temp = array();
							$metric = array();
							$metric['Metric'] = $point['metric'];
							$rate = $point['tags']['rate'];
							$desc = $point['tags']['metric_text'];
							$unit = $point['tags']['units'];
							$tags = $point['tags'];
							$metrics_host = array();
							$metrics_host_param = array();
							$metrics_host_tags = array();

							$metrics_host_tags['host'] = $point['tags']['host'];
							$metrics_host_param['tags'] = $metrics_host_tags;
							$metrics_host_param['Name'] = 'hostname';
							$metrics_host_param['Value'] = $tags['hostname'];
							$metrics_host[] = $metrics_host_param;
							$metrics_host_param['Name'] = 'polling_time';
							$metrics_host_param['Value'] = $tags['polling_time'];
							$metrics_host[] = $metrics_host_param;
							$metrics_host_param['Name'] = 'host_type';
							$metrics_host_param['Value'] = $tags['host_type'];
							$metrics_host[] = $metrics_host_param;
							$metrics[$point['tags']['host']] = $metrics_host;

							unset( $tags['metric_text'] ); // delete unwanted data
							unset( $tags['rate'] ); // delete unwanted data
							unset( $tags['namecache'] ); // delete unwanted data
							unset( $tags['type'] ); // delete unwanted data
							unset( $tags['units'] ); // delete unwanted data
							unset( $tags['host_type'] ); // delete unwanted data
							unset( $tags['hostname'] ); // delete unwanted data
							unset( $tags['polling_time'] ); // delete unwanted data
							$metric['tags'] = $tags;
							if ( isset($desc) && strlen($desc) > 0 ) {
								$metric['Name'] = 'desc';
								$metric['Value'] = $desc;
								$metrics_temp[] = $metric;
							}
							if ( isset($rate) ) {
								$metric['Name'] = 'rate';
								$metric['Value'] = $rate;
								$metrics_temp[] = $metric;
							}
							// Unit name is taken from Vertical Label of Graph Template for datasource
							// If there are multiple Graph Templates, only one value will be taken
							// Vertical Label text should be from this list, or bosun will skip it:
							// https://godoc.org/bosun.org/metadata#Unit
							if ( isset( $unit ) ) {
								$metric['Name'] = 'unit';
								$metric['Value'] = $unit;
								$metrics_temp[] = $metric;
							}
							$metrics[$point['metric']] = $metrics_temp;
						}
						if ( isset($point['tags']['index_type']) ) {
							$point['tags'][$point['tags']['index_type']] = $point['tags']['index_value'];
							unset( $point['tags']['index_type'] ); // delete unwanted data
							unset( $point['tags']['index_value'] ); // delete unwanted data
						}
						unset( $point['tags']['metric_text'] ); // delete unwanted data
						unset( $point['tags']['units'] ); // delete unwanted data
						$points[] = $point;
					break;
				}
			}
		}

		if ( sizeof( $points ) > 0 ) {
			if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_HIGH ) {
				cacti_log( "DEBUG: Adding [". sizeof($points) ."] of data points for [". $point['tags']['hostname'] ."]", TRUE, "CereusTransporter" );
			}

			switch ($db_type) {
				case 'influxdb':
					// we are writing unix timestamps, which have a seconds precision
					try {
						$newPoints = $database->writePoints($points, InfluxDB\Database::PRECISION_SECONDS);
					} catch  (Exception $e) {
						cacti_log( "ERROR: ".$e->getMessage(), TRUE, "CereusTransporter" );
					}
					break;
				case 'opentsdb':
				case 'bosun':
					$db_url = rtrim( read_config_option( 'cereus_transporter_db_fullurl' ), '/' );
					if ( $db_type == 'bosun' && sizeof($metrics) > 0 ) {
						$metrics_post = array();
						foreach( $metrics as $metrics_outer ) {
							foreach( $metrics_outer as $metric ) {
								$metrics_post[] = $metric;
							}
						}
						$json = json_encode($metrics_post);
						if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
							cacti_log( "DEBUG: Bosun Metrics for hostname [" . $point['tags']['hostname'] . "]: " . $json, TRUE, "CereusTransporter" );
						}
						#	cacti_log( "DEBUG: Bosun Metrics for hostname [" . $point['tags']['hostname'] . "]: " . $json, TRUE, "CereusTransporter" );
						$metacurl = curl_init();
						curl_setopt( $metacurl, CURLOPT_URL, $db_url.'/api/metadata/put' );
						curl_setopt( $metacurl, CURLOPT_HTTPHEADER, array( 'Content-Type: application/json; charset=UTF-8', "Accept:application/json, text/javascript, */*; q=0.01", 'Content-Length: ' . strlen($json) ) );
						curl_setopt( $metacurl, CURLOPT_RETURNTRANSFER, TRUE );
						curl_setopt( $metacurl, CURLOPT_BINARYTRANSFER, TRUE );
						curl_setopt( $metacurl, CURLOPT_POST, TRUE );
						curl_setopt( $metacurl, CURLOPT_POSTFIELDS, $json );
						$curl_result = curl_exec( $metacurl );
						if ( $curl_result === FALSE ) {
							cacti_log( "ERROR: Metadata Curl to [".$db_url."] was not succesful: [" . curl_error( $metacurl ) ."]", TRUE, "CereusTransport" );
						}
						#cacti_log( $curl_result, TRUE, "CereusTransport" );
						curl_close( $metacurl );
					}

					$json = json_encode($points);
					#	cacti_log( "DEBUG: Adding [". sizeof($points) ."] of data points for [". $point['tags']['hostname'] ."] TSDB JSON: ".json_encode($point), TRUE, "CereusTransporter" );
					if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
						cacti_log( "DEBUG: Adding [". sizeof($points) ."] of data points for [". $point['tags']['hostname'] ."] TSDB JSON: ".$json, TRUE, "CereusTransporter" );
					}
					//	cacti_log( "DEBUG: Adding [". sizeof($points) ."] of data points for [". $point['tags']['hostname'] ."] TSDB JSON: ".$json, TRUE, "CereusTransporter" );
					curl_setopt( $curl, CURLOPT_HTTPHEADER, array( 'Content-Type: application/json; charset=UTF-8', "Accept:application/json, text/javascript, */*; q=0.01", 'Content-Length: ' . strlen($json) ) );
					curl_setopt( $curl, CURLOPT_POSTFIELDS, $json );
					$curl_result = curl_exec( $curl );
					if ( $curl_result === FALSE ) {
						cacti_log( "ERROR: Data Curl to [".$db_url."] for [".$point['tags']['hostname']."] was not succesful: [" . curl_error( $curl ) ."]", TRUE, "CereusTransport" );
					}
					#	cacti_log( $curl_result, TRUE, "CereusTransport" );
					curl_close( $curl );
				break;
			}
		
			if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_HIGH ) {
				cacti_log( "Finished adding [". sizeof($points) ."] of data points for [". $point['tags']['hostname'] ."]", TRUE, "CereusTransporter" );
			}
		}

		return;
	}
	
	function CereusTransporter_poller_bottom() {
		$db_type = read_config_option( 'cereus_transporter_dbtype' );
		cacti_log( "INFO: Adding data to $db_type", TRUE, "CereusTransporter" );

		/* take time and log performance data */
		$transport_stats = array();
		$transport_stats['Time'] = ''; // init this parameter early, making it first in stats
		$start = CereusTransporter_currentTime();

		// populate field index type info
		$field_lookup_query = <<<EOT
    SELECT
     ds.id
     ,MAX(CASE WHEN type_code='index_type' THEN input.value END) index_type
     ,MAX(CASE WHEN type_code='index_value' THEN input.value END) index_value
    FROM data_template_data data
    INNER JOIN data_local ds ON ds.id=data.local_data_id
    INNER JOIN data_input_fields field ON field.data_input_id=data.data_input_id AND field.input_output='in' AND field.type_code IN ('index_type', 'index_value')
    INNER JOIN data_input_data input ON input.data_template_data_id=data.id AND input.data_input_field_id=field.id 
    WHERE 
     data.local_data_template_data_id <> 0
EOT;
		$field_lookup = db_fetch_assoc( $field_lookup_query );
		$field_info = array();
		foreach( $field_lookup as $dsfield ) {
			if ( isset( $dsfield['index_type'] ) ) {
				$field_info[$dsfield['id']]['index_type'] = CereusTransporter_cleanTag( $db_type, $dsfield['index_type'] );
				$field_info[$dsfield['id']]['index_value'] = CereusTransporter_cleanTag( $db_type, $dsfield['index_value'] );
			}
		}

		// populate lookup data - hostname, index and various info for each ds
		// if host template is empty, replacing it with the first found host template by id in db
		$ds_lookup_query = <<<EOT
    SELECT
     ds.id
     ,host.hostname
     ,host.description
     ,data.name_cache
     ,(CASE WHEN rrd.data_source_type_id=1 THEN 'gauge' WHEN rrd.data_source_type_id=2 THEN 'counter' WHEN rrd.data_source_type_id=3 THEN 'counter' WHEN rrd.data_source_type_id=4 THEN 'counter' END) AS rate
     ,data_template.name AS metric
     ,host_template.name AS host_type
     ,(CASE WHEN EXISTS (SELECT COLUMN_NAME FROM information_schema.COLUMNS WHERE TABLE_NAME='host' AND COLUMN_NAME='polling_time') THEN host.polling_time END) polling_time
    FROM data_template_data data
    INNER JOIN data_local ds ON ds.id=data.local_data_id
    INNER JOIN host ON host.id=ds.host_id
    INNER JOIN host_template ON host_template.id=(CASE host.host_template_id WHEN 0 THEN (SELECT id FROM host_template ORDER BY id LIMIT 1) ELSE host.host_template_id END)
    INNER JOIN data_template ON data_template.id=data.data_template_id 
    INNER JOIN data_template_rrd rrd ON rrd.local_data_id=data.local_data_id
    WHERE 
     data.local_data_template_data_id <> 0
     AND host.disabled <> 'on'
    GROUP BY ds.id
EOT;
		$ds_lookup = db_fetch_assoc( $ds_lookup_query );
		$ds_info = array();
		foreach( $ds_lookup as $ds ) {
			$ds_id = $ds['id'];
			$ds_info[$ds_id]['cacti_data_id'] = $ds['id'];
			$ds_info[$ds_id]['collector'] = $ds['cacti'];
			$ds_info[$ds_id]['hostname'] = $ds['hostname'];
			$host = CereusTransporter_cleanTag( $db_type, CereusTransporter_host( $ds["hostname"], $ds["description"] ) );
			$ds_info[$ds_id]['host'] = strlen($host) > 0 ? $host:$ds['hostname'];
			$ds_info[$ds_id]['host_type'] = CereusTransporter_cleanTag( $db_type, $ds['host_type'] );
			$ds_info[$ds_id]['metric'] = CereusTransporter_templateToMetric( $ds['metric'] );
			$ds_info[$ds_id]['metric_text'] = $ds['metric'];
			$ds_info[$ds_id]['rate'] = $ds['rate'];
			$ds_info[$ds_id]['namecache'] = CereusTransporter_cleanTag( $db_type, $ds['name_cache'] );
			// custom variable with host polling time info, in modified spine only
			if ( isset($ds['polling_time']) ) {
				$ds_info[$ds_id]['polling_time'] = $ds['polling_time'];
			}
			if ( isset($field_info[$ds_id]) && isset($field_info[$ds_id]['index_type']) && is_null($ds_info[$ds_id]['index_type']) ) {
				$ds_info[$ds_id]['index_type'] = $field_info[$ds_id]['index_type'];
				$ds_info[$ds_id]['index_value'] = $field_info[$ds_id]['index_value'];
			}
			// clean empty entries
			$ds_info[$ds_id] = array_filter($ds_info[$ds_id]);
		}
		$transport_stats['LookupTableSize'] = sizeof($ds_info);
		if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_HIGH ) {
			cacti_log( "DEBUG: ". $db_type ." Lookup table size: [". sizeof($ds_info) ."]", TRUE, "CereusTransporter" );
		}

		// graph requests are incredibly slow, caching units information too
		$units_lookup_query = <<<EOT
    SELECT DISTINCT
     rrd.local_data_id
     ,rrd.data_source_name
     ,gtg.vertical_label
    FROM graph_templates_item gti
    INNER JOIN data_template_rrd rrd ON rrd.id=gti.task_item_id
    INNER JOIN graph_templates_graph gtg ON gtg.local_graph_id=gti.local_graph_id
    WHERE 
     gti.local_graph_template_item_id <> 0
     AND gtg.vertical_label <> ''
EOT;
		$units_lookup = db_fetch_assoc( $units_lookup_query );
		$units = array();
		foreach($units_lookup as $unit) {
			if ( is_null( $units[$unit['local_data_id']] ) ) {
				$units[$unit['local_data_id']] = array();
			}
			$units[$unit['local_data_id']][$unit['data_source_name']] = $unit['vertical_label'];
		}
		$transport_stats['LookupUnitsTableSize'] = sizeof($units);
		if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_HIGH ) {
			cacti_log( "DEBUG: ". $db_type ." Units lookup table size: [". sizeof($units) ."]", TRUE, "CereusTransporter" );
		}

		// Retrieve data from CereusTransporter
		$transport_stats['MetricsCount'] = db_fetch_cell( "SELECT COUNT(local_data_id) FROM plugin_CereusTransporter_data" );
		$polling_data = db_fetch_assoc( "SELECT `timestamp`, `local_data_id`, `key`, `value` FROM plugin_CereusTransporter_data ORDER BY `timestamp`,`local_data_id`,`key`" );
		// Clean
		db_execute( "DELETE FROM `plugin_CereusTransporter_data`" );
		
		// Initialize data 
		$old_hostname = '';
		$data_array = array();
		$timestamp = '';
		$host_name = '';
		$host_times = array();

		$host_start_time = CereusTransporter_currentTime();

		foreach ( $polling_data as $item ) {
			if ( $item[ 'timestamp' ] > 0 ) {
				$key = $item[ 'key' ];
				$val = $item[ 'value' ];
				$timestamp = $item[ 'timestamp' ];
				$id = $item[ 'local_data_id' ];
				if ( is_numeric( $val ) ) {
					$host_name = $ds_info[$id]['hostname'];
					if ($old_hostname <> $host_name) {
						if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
							cacti_log( "DEBUG: Adding the following data to $db_type: [".$timestamp."] [".$old_hostname."]", TRUE, "CereusTransporter" );
						}

						if ( strlen($host_name) < 1 ) {
							cacti_log( "Empty hostname for datasource #[".$id."]", TRUE, "CereusTransporter" );
						}

						if ( strlen($old_hostname) > 0 ) {	
							// Send data for old host:
							CereusTransporter_send_data($data_array);
						}						

						// Reset/Initialize variables
						$host_times[$old_hostname] = round( CereusTransporter_currentTime() - $host_start_time, 4 );
						if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
							cacti_log( "DEBUG: Finished adding the data to $db_type: [".$old_hostname."] [Time: ".$host_times[$old_hostname]."]", TRUE, "CereusTransporter" );
						}

						$host_start_time = CereusTransporter_currentTime();
						$old_hostname = $host_name;
						$data_array = array();
					}
					$point['metric'] = $ds_info[$id]['metric'];
					$point['timestamp'] = (int)$timestamp;
					$point['value'] = (int)$val;
					$point['tags'] = $ds_info[$id];
					$point['tags']['type'] = $key;
					$unit = null;
					// add units info
					if ( isset( $units[$id] ) && isset( $units[$id][$key] ) ) {
						$point['tags']['units'] = $units[$id][$key];
					}

					unset( $point['tags']['metric'] ); // delete unwanted data
					unset( $point['tags']['id'] ); // delete unwanted data

					// push point to array unconditionally, to avoid skipping data
					$data_array[] = $point;
				}
			}
		}

		// Send last data item:
		if ( read_config_option( 'log_verbosity' ) >= POLLER_VERBOSITY_DEBUG ) {
			cacti_log( "DEBUG: Adding the following data to ".$db_type.": [".$timestamp."] [".$host_name."]", TRUE, "CereusTransporter" );
		}
		// Send data to database:
		CereusTransporter_send_data($data_array);
		$host_times[$old_hostname] = round( CereusTransporter_currentTime() - $host_start_time, 2 );

		// log performance data
		$transport_stats['Time'] = sprintf( "%01.2f", CereusTransporter_currentTime() - $start );
		$transport_stats['AverageHostTime'] = sprintf( "%01.2f", array_sum($host_times) / count($host_times) );
		asort( $host_times );
		$slowest_hosts = array_slice( $host_times, -4 );
		arsort($slowest_hosts);
		$transport_stats['SlowestHostnames'] = '[';
		foreach( $slowest_hosts as $hostname => $time ) {
			$transport_stats['SlowestHostnames'] .= $hostname . ': ' . sprintf( "%01.3f", $time ) . 's, ';
		}
		$transport_stats['SlowestHostnames'] = rtrim($transport_stats['SlowestHostnames'], ', ') . ']';
		$transport_stats_text = '';
		foreach( $transport_stats as $data => $value ) {
			$transport_stats_text .= $data . ':' . $value . ' ';
		}
		$transport_stats_text = rtrim($transport_stats_text, '; ');
		cacti_log( "STATS: " . $transport_stats_text, TRUE, "CereusTransporter" );
	}
	
	

	

	/**
	 * Returns strings with space, comma, or equals sign characters backslashed per Influx write protocol syntax
	 *
	 * @param string $value
	 * @return string
	 */
	function CereusTransporter_addShalshes($value) {
		$value = str_replace(' ','\ ',$value);
		$value = str_replace(',','\,',$value);
		$value = str_replace('=','\=',$value);
		return $value;
	}

	/**
	 * Cleans white space and unprintable characters from string
	 *
	 * @param string $db_type
	 * @param string $value
	 * @return string
	 */
	function CereusTransporter_cleanTag($db_type, $value) {
		if ( $db_type == 'influxdb' ) {
			return CereusTransporter_addShalshes($value);
		}
		$value = trim($value);
		$value = preg_replace("/[^A-Za-z0-9_\/\-]+/", '_', $value);
		$value = trim($value, '_');
		return $value;
	}

	/**
	 * Finds host name by hostname and host description ('10.10.10.10 Hostname' and 'Hostname 10.10.10.10' formats)
	 *
	 * @param string $hostname
	 * @param string $description
	 * @return string
	 */
	function CereusTransporter_host($hostname, $description) {
		if (filter_var($hostname, FILTER_VALIDATE_IP) && strpos($description, $hostname) !== false) {
			return preg_replace('/\s*'.preg_quote($hostname, '/').'\s*/', '', $description);
		}
		return $hostname;
	}

	/**
	 * Replaces Data Template name with like.this.notation
	 *
	 * @param string $value
	 * @return string
	 */
	function CereusTransporter_templateToMetric($value) {
		$value = trim($value);
		$value = preg_replace("/[^A-Za-z0-9_\.]+/", '.', $value);
		$value = rtrim($value, '.');
		$value = strtolower($value);
		return $value;
	}

	function CereusTransporter_currentTime() {
		list( $micro, $seconds ) = explode( " ", microtime() );
		return $seconds + $micro;
	}

	/**
	 *
	 */
	function plugin_CereusTransporter_uninstall()
	{
		// Do any extra Uninstall stuff here
		return;
	}


	/**
	 * @return bool
	 */
	function plugin_CereusTransporter_check_config()
	{
		// Here we will check to ensure everything is configured
		CereusTransporter_check_upgrade();

		return TRUE;
	}

	/**
	 * @return bool
	 */
	function plugin_CereusTransporter_upgrade()
	{
		// Here we will upgrade to the newest version
		CereusTransporter_check_upgrade();
		return FALSE;
	}

	/**
	 * @return array
	 */
	function plugin_CereusTransporter_version()
	{
		return CereusTransporter_version();
	}

	/**
	 *
	 */
	function CereusTransporter_check_upgrade()
	{
		global $config;

		$files = array('index.php', 'plugins.php');
		if (isset($_SERVER['PHP_SELF']) && !in_array(basename($_SERVER['PHP_SELF']), $files)) {
			return;
		}

		$current = CereusTransporter_version();
		$current = $current['version'];
		$old     = db_fetch_row("SELECT * FROM plugin_config WHERE directory='CereusTransporter'");
		if (sizeof($old) && $current != $old["version"]) {
			/* if the plugin is installed and/or active */
			if ($old["status"] == 1 || $old["status"] == 4) {
				/* re-register the hooks */
				plugin_CereusTransporter_install();

				/* perform a database upgrade */
				if ($old['version'] < 0.45) {
					db_execute( "DROP TABLE `plugin_CereusTransporter_data`" );
					CereusTransporter_setup_table_new();
				}
			}

			/* update the plugin information */
			$info = CereusTransporter_version();
			$id   = db_fetch_cell("SELECT id FROM plugin_config WHERE directory='CereusTransporter'");
			db_execute("UPDATE plugin_config
				SET name='" . $info["longname"] . "',
				author='"   . $info["author"]   . "',
				webpage='"  . $info["homepage"] . "',
				version='"  . $info["version"]  . "'
				WHERE id='$id'");
		}

		return;
	}

	/**
	 * @return bool
	 */
	function CereusTransporter_check_dependencies()
	{
		return TRUE;
	}

	function CereusTransporter_config_settings()
	{
		global $tabs, $settings;

		$temp = array(
			"cereus_transporter_header"       => array(
				"friendly_name" => "CereusTransporter Settings",
				"method"        => "spacer",
			),
			"cereus_transporter_db_fullurl"          => array(
				"friendly_name" => "Target URL/System",
				"description"   => "Full Database URL. Example: https://user:password@localhost:8086/dbname for InfluxDB, https://user:password@localhost:8070/ for Bosun",
				"method"        => "textbox",
				"default"	=> "",
				"max_length"    => 1024,
			),
			"cereus_transporter_dbtype" => array(
				"friendly_name" => "Database target type",
				"description" => "The type of database being used for exporting the data to.",
				"method" => "drop_array",
				"default" => "influxdb",
				"array" => array(
					"influxdb" => "InfluxDB",
					"opentsdb" => "OpenTSDB",
					"bosun" => "Bosun"
				)
			)
		);

		if ( isset( $settings[ "misc" ] ) ) {
			$settings[ "misc" ] = array_merge( $settings[ "misc" ], $temp );
		}
		else {
			$settings[ "misc" ] = $temp;
		}
	}


	/**
	 * @return array
	 */
	function CereusTransporter_version()
	{
		return array(
			'name'     => 'CereusTransporter',
			'version'  => '0.45',
			'longname' => 'CereusTransporter Plugin',
			'author'   => 'Thomas Urban',
			'homepage' => 'http://blog.network-outsourcing.de/products/nmid-plugins/cereustransporter',
			'email'    => 'sales@urban-software.de',
			'url'      => 'http://www.urban-software.de/versions.php'
		);
	}

	/**
	 *
	 */
	function CereusTransporter_console_after()
	{
		return;
	}

	/**
	 *
	 */
	function CereusTransporter_setup_table()
	{
		return;
	}
	
	function CereusTransporter_setup_table_new() {
		global $config, $database_default;
		include_once($config["library_path"] . "/database.php");
    
		$data = array();
		$data['columns'][] = array('name' => 'timestamp', 'type' => 'varchar(1024)', 'NULL' => false, 'default' => '0');
		$data['columns'][] = array('name' => 'local_data_id', 'type' => 'int(11)', 'NULL' => false, 'default' => '0');
		$data['columns'][] = array('name' => 'key', 'type' => 'varchar(1024)', 'NULL' => false, 'default' => '0');
		$data['columns'][] = array('name' => 'value', 'type' => 'varchar(1024)', 'NULL' => false, 'default' => '0');
		$data['keys'][] = array('name' => 'local_data_id', 'columns' => 'local_data_id');
		$data['keys'][] = array('name' => 'key', 'columns' => 'key');
		$data['type'] = 'Memory';
		$data['comment'] = 'NMID CereusTransporter Data';
		api_plugin_db_table_create('CereusTransporter', 'plugin_CereusTransporter_data', $data);
	}
